/**
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights Reserved.
 */
package com.topcoder.rules.dependencymap.view;

import java.io.File;
import com.topcoder.util.image.svg2dgraphics.SVGGraphics2D;

/**
 * The SVG Renderer will implement the DiagramRenderer and will render SVG diagrams to a file. This renderer can
 * operate in two modes depending on the path passed in the constructor.
 *
 * 1) Render to a specific file. If the path specifies a specific file, then the rendered diagram will go directly
 * to that file. This allows the application to be able to finely specify where the rendering goes to.
 * 2) Render to a specific directory.  If the path specifies a specific directory, then the rendered diagram will
 * go into that directory using the dependency's ID as the name of the file. This allows the application to render
 * multiple diagrams without having to specify the file names for each.
 *
 * @author Pops
 * @author TCSDeveloper
 */
public class SVGRenderer implements DiagramRenderer {

    /**
     * This variable represents the path (either a specific file or directory) that the diagram will be rendered to.
     * The variable is initially set in the constructor, is mutatable via the setter method and will be accessed by
     * the getter method and the disposeOf methods.
     */
    private File path;

    /**
     * Constructs the SVG Renderer using the specified path. This method simply saves the path to the like named
     * variable.
     *
     * @param path The file or directory to render to
     * @throws NullPointerException if the path is null
     */
    public SVGRenderer(File path) {
        this.setPath(path);
    }

    /**
     * This is the getter method for the path variable.
     *
     * @return The file/directory that will be rendered to.
     */
    public File getPath() {
        return this.path;
    }

    /**
     * This method is simply the setter method for the path variable.
     *
     * @param path Either the file or directory to render to
     * @throws NullPointerException if path is null
     */
    public void setPath(File path) {
        if (path == null) {
            throw new NullPointerException("path should not be null.");
        }
        this.path = path;
    }

    /**
     * This method will:
     *
     * 1) If the path specifies a file, create the file (including any part of the directory structure that
     * doesn't exist) if it doesn't exist.
     * 2) If the path specifies a directory, create a new file from the directory with the dependency.getTitle()
     * as the filename. Create the file (including any part of the directory structure that doesn't exist).
     * 3) Create and return a new SVGGraphicsAdapater(filename)
     *
     * @param dependency The dependency that will be rendered
     * @param width The width of the diagram (not used)
     * @param height The height of the diagram (not used)
     * @return A graphics object to render with
     * @throws NullPointerException if dependency is null
     * @throws IllegalArgumentException if the width/height is <= 0 (even if it's not used)
     * @throws RendererException if an error occurs getting the graphics
     */
    public RendererGraphics getGraphics(Dependency dependency,  int width, int height ) throws RendererException {
        if (dependency == null) {
            throw new NullPointerException("dependency should not be null.");
        }

        File file = this.path;
        if (file.isDirectory()) {
            file = new File(file, dependency.getTitle() + ".svg");
        }

        return new SVGGraphics2DAdapter(new SVGGraphics2D(), file.getAbsolutePath());
    }

}
