/**
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights Reserved.
 */
package com.topcoder.rules.dependencymap.view;

import junit.framework.TestCase;
import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.awt.Color;
import java.awt.Font;
import com.topcoder.util.image.svg2dgraphics.SVGGraphics2D;

/**
 * Unit test cases for SVGGraphicsAdapter.
 *
 * Testing of SVGGraphicsAdpater verifies the adapter interacts properly with the SVG component.
 *
 * @author TCSDeveloper
 */
public class SVGGraphicsAdapterUnitTests extends TestCase {

    /**
     * The color to use.
     */
    private Color color = null;

    /**
     * The font to use.
     */
    private Font font = null;

    /**
     * The filename to persist to.
     */
    private String filename = "test_files/unittests/unittest-";

    /**
     * The SVGGraphicsAdapter instance to test.
     */
    private SVGGraphics2DAdapter adapter = null;

    /**
     * Create a new adapter for each test.
     */
    protected void setUp() {
        adapter = new SVGGraphics2DAdapter(new SVGGraphics2D(), filename);
    	color = adapter.getColor();
        font = adapter.getFont();
    }

    /**
     * Remove the file and dispose the adapter.
     */
    protected void tearDown() {
        if (adapter != null) {
            adapter.dispose();
        }
    }

    /**
     * Create instance with filename.
     */
    public void testCreate_Filename() {
        adapter.dispose();

        adapter = new SVGGraphics2DAdapter(new SVGGraphics2D(), filename + "testcreate.svg");
    }

    /**
     * Create instance with null filename.
     */
    public void testCreate_NullFilename() {
        try {
            new SVGGraphics2DAdapter(new SVGGraphics2D(), null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Create instance with empty filename.
     */
    public void testCreate_EmptyFilename() {
        try {
            new SVGGraphics2DAdapter(new SVGGraphics2D(), " ");
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Test setColor().
     */
    public void testSetColor() {
        adapter.setColor(color);
        assertEquals("Color mismatches.", color, adapter.getColor());
    }

    /**
     * Test setColor() with null color.
     */
    public void testSetColor_NullColor() {
        try {
            adapter.setColor(null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Test getColor().
     */
    public void testGetColor() {
        adapter.setColor(color);
        assertEquals("Color mismatches.", color, adapter.getColor());
    }

    /**
     * Test setFont().
     */
    public void testSetFont() {
        adapter.setFont(font);
        assertEquals("Font mismatches.", font, adapter.getFont());
    }

    /**
     * Test setFont() with null font.
     */
    public void testSetFont_NullFont() {
        try {
            adapter.setFont(null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Test getFont().
     */
    public void testGetFont() {
        adapter.setFont(font);
        assertEquals("Font mismatches.", font, adapter.getFont());
    }

    /**
     * Test getFontMetrics().
     */
    public void testGetFontMetrics() {
        assertNotNull("FontMetrics not returned.", adapter.getFontMetrics(font));
        assertEquals("Font mismatches.", font, adapter.getFontMetrics(font).getFont());
    }

    /**
     * Test getFontMetrics() with null font.
     */
    public void testGetFontMetrics_NullFont() {
        try {
            adapter.getFontMetrics(null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Test draw().
     *
     * @throws RendererException to JUnit.
     */
    public void testDraw() throws RendererException {
    	adapter = new SVGGraphics2DAdapter(new SVGGraphics2D(), filename);
        adapter.draw(new ShapeFactory().getShape(ShapeFactory.RELATIONSHIP_ARROW));              
        adapter.complete();
    }

    /**
     * Test draw() with null shape.
     */
    public void testDraw_NullShape() {
        try {
            adapter.draw(null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Test drawString().
     *
     * @throws RendererException to JUnit.
     */
    public void testDrawString() throws RendererException {
    	adapter = new SVGGraphics2DAdapter(new SVGGraphics2D(), filename + "string.svg");
        adapter.drawString("Dependency Map Renderer", 15, 23);
        adapter.complete();
    }

    /**
     * Test drawString() with null str.
     */
    public void testDrawString_NullShape() {
        try {
            adapter.drawString(null, 18, 23);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Test complete().
     *
     * @throws RendererException to JUnit.
     */
    public void testComplete() throws RendererException {
    	adapter = new SVGGraphics2DAdapter(new SVGGraphics2D(), filename + "-create.svg");
        adapter.complete();
        assertTrue("Output does not exist.", new File(filename).exists());
    }

    /**
     * Test dispose().
     */
    public void testDispose() {
        adapter.dispose();
        adapter = null;
    }

}
